@@ -3,10 +3,10 @@
   array.c -
 
   $Author: matz $
-  $Date: 1999/10/18 09:07:13 $
+  $Date: 2000/03/23 04:14:07 $
   created at: Fri Aug  6 09:46:12 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -663,7 +663,7 @@ rb_ary_join(ary, sep)
 	}
 	break;
       default:
-	result = rb_obj_as_string(tmp);
+	result = rb_str_dup(rb_obj_as_string(tmp));
 	break;
     }
 
