@@ -3,7 +3,7 @@
   bignum.c -
 
   $Author: matz $
-  $Date: 1999/12/08 05:35:54 $
+  $Date: 2000/04/14 14:35:40 $
   created at: Fri Jun 10 00:48:55 JST 1994
 
 ************************************************/
@@ -16,9 +16,9 @@ VALUE rb_cBignum;
 typedef unsigned short USHORT;
 
 #define BDIGITS(x) RBIGNUM(x)->digits
-#define BITSPERDIG (sizeof(short)*CHAR_BIT)
+#define BITSPERDIG (sizeof(USHORT)*CHAR_BIT)
 #define BIGRAD (1L << BITSPERDIG)
-#define DIGSPERINT ((unsigned int)(sizeof(long)/sizeof(short)))
+#define DIGSPERLONG ((unsigned int)(sizeof(long)/sizeof(USHORT)))
 #define BIGUP(x) ((unsigned long)(x) << BITSPERDIG)
 #define BIGDN(x) RSHIFT(x,BITSPERDIG)
 #define BIGLO(x) ((USHORT)((x) & (BIGRAD-1)))
@@ -33,7 +33,7 @@ bignew_1(klass, len, sign)
     OBJSETUP(big, klass, T_BIGNUM);
     big->sign = sign;
     big->len = len;
-    BDIGITS(big) = ALLOC_N(USHORT, len);
+    big->digits = ALLOC_N(USHORT, len);
 
     return (VALUE)big;
 }
@@ -69,7 +69,7 @@ rb_big_2comp(x)			/* get 2's complement */
 	for (i=1; i<RBIGNUM(x)->len; i++) {
 	    if (ds[i] != 0) return;
 	}
-	REALLOC_N(BDIGITS(x), USHORT, RBIGNUM(x)->len++);
+	REALLOC_N(RBIGNUM(x)->digits, USHORT, RBIGNUM(x)->len++);
 	ds = BDIGITS(x);
 	ds[RBIGNUM(x)->len-1] = 1;
     }
@@ -116,14 +116,14 @@ rb_uint2big(n)
     VALUE big;
 
     i = 0;
-    big = bignew(DIGSPERINT, 1);
+    big = bignew(DIGSPERLONG, 1);
     digits = BDIGITS(big);
-    while (i < DIGSPERINT) {
+    while (i < DIGSPERLONG) {
 	digits[i++] = BIGLO(n);
 	n = BIGDN(n);
     }
 
-    i = DIGSPERINT;
+    i = DIGSPERLONG;
     while (i-- && !digits[i]) ;
     RBIGNUM(big)->len = i+1;
     return big;
