@@ -3,10 +3,10 @@
   dir.c -
 
   $Author: matz $
-  $Date: 1999/11/24 07:13:50 $
+  $Date: 2000/02/17 08:28:51 $
   created at: Wed Jan  5 09:51:01 JST 1994
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -510,7 +510,7 @@ extract_path(p, pend)
     len = pend - p;
     alloc = ALLOC_N(char, len+1);
     memcpy(alloc, p, len);
-    if (len > 0 && pend[-1] == '/') {
+    if (len > 1 && pend[-1] == '/') {
 	alloc[len-1] = 0;
     }
     else {
@@ -576,11 +576,12 @@ glob(path, func, arg)
 		break;
 	    }
 	    magic = extract_elem(p);
+#define BASE (*base && !(*base == '/' && !base[1]))
+
 	    for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
 		if (fnmatch(magic, dp->d_name, FNM_PERIOD|FNM_PATHNAME) == 0) {
 		    char *fix = ALLOC_N(char, strlen(base)+NAMLEN(dp)+2);
-
-		    sprintf(fix, "%s%s%s", base, (*base)?"/":"", dp->d_name);
+		    sprintf(fix, "%s%s%s", base, (BASE)?"/":"", dp->d_name);
 		    if (!m) {
 			(*func)(fix, arg);
 			free(fix);
