@@ -3,10 +3,10 @@
   enum.c -
 
   $Author: matz $
-  $Date: 1999/12/07 09:24:55 $
+  $Date: 2000/04/10 05:48:27 $
   created at: Fri Oct  1 15:15:19 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -92,7 +92,7 @@ enum_find(argc, argv, obj)
 	return arg.val;
     }
     if (!NIL_P(if_none)) {
-	rb_eval_cmd(if_none, Qnil);
+	rb_eval_cmd(if_none, rb_ary_new2(0));
     }
     return Qnil;
 }
