@@ -3,6 +3,8 @@ require 'mkmf'
 dir_config("dbm")
 if have_library("gdbm", "dbm_open")
   gdbm = true
+else
+  gdbm = false
 end
 gdbm or have_library("db", "dbm_open") or have_library("dbm", "dbm_open")
 have_header("cdefs.h") 
