@@ -1,7 +1,14 @@
 require 'nkf'
 
 module Kconv
-  def kconv(str, out_code, in_code = NKF::AUTO)
+  AUTO = NKF::AUTO
+  JIS = NKF::JIS
+  EUC = NKF::EUC
+  SJIS = NKF::SJIS
+  BINARY = NKF::BINARY
+  NOCONV = NKF::NOCONV
+  UNKNOWN = NKF::UNKNOWN
+  def kconv(str, out_code, in_code = AUTO)
     opt = '-'
     case in_code
     when NKF::JIS
@@ -25,27 +32,27 @@ module Kconv
 
     opt = '' if opt == '-'
 
-    return NKF::nkf(opt, str)
+    NKF::nkf(opt, str)
   end
   module_function :kconv
 
   def tojis(str)
-    return NKF::nkf('-j', str)
+    NKF::nkf('-j', str)
   end
   module_function :tojis
 
   def toeuc(str)
-    return NKF::nkf('-e', str)
+    NKF::nkf('-e', str)
   end
   module_function :toeuc
 
   def tosjis(str)
-    return NKF::nkf('-s', str)
+    NKF::nkf('-s', str)
   end
   module_function :tosjis
 
   def guess(str)
-    return NKF::guess(str)
+    NKF::guess(str)
   end
   module_function :guess
 end
