@@ -6,6 +6,7 @@ case PLATFORM
 when /mswin32/
   test_func = "WSACleanup"
   have_library("wsock32", "WSACleanup")
+  have_func("closesocket")
 when /cygwin/
 #  $LDFLAGS << " -L/usr/lib" if File.directory?("/usr/lib")
 #  $CFLAGS << " -I/usr/include"
@@ -14,6 +15,7 @@ when /cygwin/
 when /beos/
   test_func = "socket"
   have_library("net", "socket")
+  have_func("closesocket")
 when /i386-os2_emx/
   test_func = "socket"
   have_library("socket", "socket")
@@ -41,6 +43,7 @@ end
 $ipv6type = nil
 $ipv6lib = nil
 $ipv6libdir = nil
+$ipv6trylibc = nil
 if $ipv6
   if egrep_cpp("yes", <<EOF)
 #include <netinet/in.h>
@@ -59,6 +62,7 @@ EOF
     $ipv6type = "kame"
     $ipv6lib="inet6"
     $ipv6libdir="/usr/local/v6/lib"
+    $ipv6trylibc=true
     $CFLAGS="-DINET6 "+$CFLAGS
   elsif File.directory? "/usr/inet6"
     $ipv6type = "linux"
@@ -100,7 +104,7 @@ EOF
   if $ipv6lib
     if File.directory? $ipv6libdir and File.exist? "#{$ipv6libdir}/lib#{$ipv6lib}.a"
       $LOCAL_LIBS = " -L#$ipv6libdir -l#$ipv6lib"
-    else
+    elsif !$ipv6trylibc
       print <<EOS
 
 Fatal: no #$ipv6lib library found.  cannot continue.
@@ -135,6 +139,23 @@ end
 #include <netdb.h>
 #include <string.h>
 #include <sys/socket.h>
+int
+main()
+{
+   struct sockaddr_storage ss;
+
+   ss.ss_family;
+   return 0;
+}
+EOF
+    $CFLAGS="-DHAVE_SOCKADDR_STORAGE "+$CFLAGS
+end
+
+  if try_link(<<EOF)
+#include <sys/types.h>
+#include <netdb.h>
+#include <string.h>
+#include <sys/socket.h>
 #include <netinet/in.h>
 int
 main()
@@ -225,7 +246,9 @@ main()
     }
   }
 
-  if (inet6 != 2 || inet4 != 2)
+  if (!(inet4 == 0 || inet4 == 2))
+    goto bad;
+  if (!(inet6 == 0 || inet6 == 2))
     goto bad;
 
   if (aitop)
