@@ -38,6 +38,7 @@
  * - PF_UNSPEC case would be handled in getipnodebyname() with the AI_ALL flag.
  */
 
+#include "config.h"
 #include <sys/types.h>
 #ifndef NT
 #include <sys/param.h>
@@ -66,7 +67,6 @@
 #include <socks.h>
 #endif
 
-#include "config.h"
 #include "addrinfo.h"
 #include "sockport.h"
 
@@ -130,7 +130,6 @@ static struct afd {
 #define PTON_MAX	4
 #endif
 
-
 static int get_name __P((const char *, struct afd *,
 			  struct addrinfo **, char *, struct addrinfo *,
 			  int));
@@ -422,7 +421,11 @@ getaddrinfo(hostname, servname, hints, res)
 			s = socket(afd->a_af, SOCK_DGRAM, 0);
 			if (s < 0)
 				continue;
+#if defined(HAVE_CLOSESOCKET)
+			closesocket(s);
+#else
 			close(s);
+#endif
 
 			if (pai->ai_flags & AI_PASSIVE) {
 				GET_AI(cur->ai_next, afd, afd->a_addrany, port);
@@ -575,11 +578,6 @@ get_addr(hostname, af, res, pai, port0)
 	struct afd *afd;
 	int i, error = 0, h_error;
 	char *ap;
-#ifndef INET6
-#ifndef NT
-	extern int h_errno;
-#endif
-#endif
 
 	top = NULL;
 	sentinel.ai_next = NULL;
