@@ -3,7 +3,7 @@
   socket.c -
 
   $Author: matz $
-  $Date: 1999/12/07 16:47:18 $
+  $Date: 2000/04/14 14:35:45 $
   created at: Thu Mar 31 12:21:29 JST 1994
 
 ************************************************/
@@ -81,10 +81,27 @@ int Rconnect();
 #define INET_SERVER 1
 #define INET_SOCKS  2
 
-#ifndef INET6
-# undef  ss_family
-# define sockaddr_storage	sockaddr
-# define ss_family		sa_family
+#ifndef HAVE_SOCKADDR_STORAGE
+/*
+ * RFC 2553: protocol-independent placeholder for socket addresses
+ */
+#define _SS_MAXSIZE	128
+#define _SS_ALIGNSIZE	(sizeof(double))
+#define _SS_PAD1SIZE	(_SS_ALIGNSIZE - sizeof(unsigned char) * 2)
+#define _SS_PAD2SIZE	(_SS_MAXSIZE - sizeof(unsigned char) * 2 - \
+				_SS_PAD1SIZE - _SS_ALIGNSIZE)
+
+struct sockaddr_storage {
+#ifdef HAVE_SA_LEN
+	unsigned char ss_len;		/* address length */
+	unsigned char ss_family;	/* address family */
+#else
+	unsigned short ss_family;
+#endif
+	char	__ss_pad1[_SS_PAD1SIZE];
+	double	__ss_align;	/* force desired structure storage alignment */
+	char	__ss_pad2[_SS_PAD2SIZE];
+};
 #endif
 
 #ifdef NT
@@ -93,16 +110,12 @@ sock_finalize(fptr)
     OpenFile *fptr;
 {
     SOCKET s;
-	extern int errno;
 
     if (!fptr->f) return;
-
-	myfdclose(fptr->f);
-	if(fptr->f2)  myfdclose(fptr->f);
-/*
-	s = get_osfhandle(fileno(fptr->f));
+    s = get_osfhandle(fileno(fptr->f));
+    myfdclose(fptr->f);
+    if (fptr->f2) myfdclose(fptr->f2);
     closesocket(s);
-*/
 }
 #endif
 
@@ -172,7 +185,7 @@ bsock_close_read(sock)
     rb_thread_fd_close(fileno(fptr->f));
     fptr->mode &= ~FMODE_READABLE;
 #ifdef NT
-    free(fptr->f);
+    myfdclose(fptr->f);
 #else
     fclose(fptr->f);
 #endif
@@ -196,7 +209,7 @@ bsock_close_write(sock)
     shutdown(fileno(fptr->f2), 1);
     fptr->mode &= ~FMODE_WRITABLE;
 #ifdef NT
-    free(fptr->f2);
+    myfdclose(fptr->f2);
 #else
     fclose(fptr->f2);
 #endif
@@ -378,8 +391,8 @@ s_recv(sock, argc, argv, from)
     GetOpenFile(sock, fptr);
     fd = fileno(fptr->f);
     rb_thread_wait_fd(fd);
-    TRAP_BEG;
   retry:
+    TRAP_BEG;
     RSTRING(str)->len = recvfrom(fd, RSTRING(str)->ptr, RSTRING(str)->len, flags,
 				 (struct sockaddr*)buf, &alen);
     TRAP_END;
@@ -498,6 +511,7 @@ ip_addrsetup(host, port)
 	long i = NUM2LONG(host);
 
 	mkinetaddr(htonl(i), hbuf, sizeof(hbuf));
+	hostp = hbuf;
     }
     else {
 	char *name = STR2CSTR(host);
@@ -508,11 +522,14 @@ ip_addrsetup(host, port)
 	else if (name[0] == '<' && strcmp(name, "<broadcast>") == 0) {
 	    mkinetaddr(INADDR_BROADCAST, hbuf, sizeof(hbuf));
 	}
+	else if (strlen(name) > sizeof(hbuf)-1) {
+	    rb_raise(rb_eSocket, "hostname too long (%d)", strlen(name));
+	}
 	else {
 	    strcpy(hbuf, name);
 	}
+	hostp = hbuf;
     }
-    hostp = hbuf;
     if (NIL_P(port)) {
 	portp = 0;
     }
@@ -529,6 +546,9 @@ ip_addrsetup(host, port)
     hints.ai_socktype = SOCK_DGRAM;
     error = getaddrinfo(hostp, portp, &hints, &res);
     if (error) {
+	if (hostp && hostp[strlen(hostp)-1] == '\n') {
+	    rb_raise(rb_eSocket, "newline at the end of hostname");
+	}
 	rb_raise(rb_eSocket, "%s", gai_strerror(error));
     }
 
@@ -742,7 +762,11 @@ open_inet(class, h, serv, type)
 	}
 
 	if (status < 0) {
+#if defined(HAVE_CLOSESOCKET)
+	    closesocket(fd);
+#else
 	    close(fd);
+#endif
 	    fd = -1;
 	    continue;
 	} else
@@ -750,7 +774,11 @@ open_inet(class, h, serv, type)
     }
     if (status < 0) {
 	if (fd >= 0)
+#if defined(HAVE_CLOSESOCKET)
+	    closesocket(fd);
+#else
 	    close(fd);
+#endif
 	freeaddrinfo(res0);
 	rb_sys_fail(syscall);
     }
@@ -1013,7 +1041,11 @@ open_unix(class, path, server)
     }
 
     if (status < 0) {
+#if defined(HAVE_CLOSESOCKET)
+	closesocket(fd);
+#else
 	close(fd);
+#endif
 	rb_sys_fail(sockaddr.sun_path);
     }
 
