@@ -1,6 +1,6 @@
 #
 #		tktext.rb - Tk text classes
-#			$Date: 1999/08/24 08:21:56 $
+#			$Date: 2000/04/10 05:56:59 $
 #			by Yukihiro Matsumoto <matz@caelum.co.jp>
 
 require 'tk.rb'
@@ -28,8 +28,7 @@ module TkTreatTextTagFont
     else
       pathname = self.path + ';' + tag
     end
-    if (fnt = slot['font'])
-      slot['font'] = nil
+    if (fnt = slot.delete('font'))
       if fnt.kind_of? TkFont
 	return fnt.call_font_configure(pathname, 
 				       self.path,'tag','configure',tag,slot)
@@ -37,16 +36,13 @@ module TkTreatTextTagFont
 	latintagfont_configure(tag, fnt) if fnt
       end
     end
-    if (ltn = slot['latinfont'])
-      slot['latinfont'] = nil
+    if (ltn = slot.delete('latinfont'))
       latintagfont_configure(tag, ltn) if ltn
     end
-    if (ltn = slot['asciifont'])
-      slot['asciifont'] = nil
+    if (ltn = slot.delete('asciifont'))
       latintagfont_configure(tag, ltn) if ltn
     end
-    if (knj = slot['kanjifont'])
-      slot['kanjifont'] = nil
+    if (knj = slot.delete('kanjifont'))
       kanjitagfont_configure(tag, knj) if knj
     end
 
@@ -127,30 +123,48 @@ end
 
 class TkText<TkTextWin
   include TkTreatTextTagFont
+  include Scrollable
 
   WidgetClassName = 'Text'.freeze
   WidgetClassNames[WidgetClassName] = self
+
   def self.to_eval
     WidgetClassName
   end
-  include Scrollable
-  def create_self
-    tk_call 'text', @path
+
+  def self.new(*args, &block)
+    obj = super(*args){}
+    obj.init_instance_variable
+    obj.instance_eval &block if defined? yield
+    obj
+  end
+
+  def init_instance_variable
     @tags = {}
   end
+
+  def create_self
+    tk_call 'text', @path
+    init_instance_variable
+  end
+
   def index(index)
     tk_send 'index', index
   end
+
   def value
     tk_send 'get', "1.0", "end - 1 char"
   end
+
   def value= (val)
     tk_send 'delete', "1.0", 'end'
     tk_send 'insert', "1.0", val
   end
+
   def _addcmd(cmd)
     @cmdtbl.push cmd
   end
+
   def _addtag(name, obj)
     @tags[name] = obj
   end
@@ -168,11 +182,19 @@ class TkText<TkTextWin
       tagid2obj(elt)
     }
   end
+
+  def mark_names
+    tk_split_list(tk_send('mark', 'names')).collect{|elt|
+      tagid2obj(elt)
+    }
+  end
+
   def window_names
     tk_send('window', 'names').collect{|elt|
       tagid2obj(elt)
     }
   end
+
   def image_names
     tk_send('image', 'names').collect{|elt|
       tagid2obj(elt)
@@ -182,6 +204,7 @@ class TkText<TkTextWin
   def set_insert(index)
     tk_send 'mark', 'set', 'insert', index
   end
+
   def set_current(index)
     tk_send 'mark', 'set', 'current', index
   end
@@ -191,6 +214,7 @@ class TkText<TkTextWin
   end
 
   def destroy
+    @tags = {} unless @tags
     @tags.each_value do |t|
       t.destroy
     end
@@ -403,19 +427,160 @@ class TkText<TkTextWin
   def rsearch(pat,start,stop=None)
     rsearch_with_length(pat,start,stop)[0]
   end
+
+  def dump(type_info, *index)
+    args = type_info.collect{|inf|
+                 if inf.kind_of? Array
+                   inf[0] = '-' + inf[0]
+                   inf
+                 else
+                   '-' + inf
+                 end
+               }.flatten
+    str = tk_send('dump', *(args + index))
+    result = []
+    sel = nil
+    i = 0
+    while i < str.size
+      # retrieve key
+      idx = str.index(/ /, i)
+      result.push str[i..(idx-1)]
+      i = idx + 1
+      
+      # retrieve value
+      case result[-1]
+      when 'text'
+	if str[i] == ?{
+	  # text formed as {...}
+	  val, i = _retrieve_braced_text(str, i)
+	  result.push val
+	else
+	  # text which may contain backslahes
+	  val, i = _retrieve_backslashed_text(str, i)
+	  result.push val
+	end
+      else
+	idx = str.index(/ /, i)
+	val = str[i..(idx-1)]
+	case result[-1]
+	when 'mark'
+	  case val
+	  when 'insert'
+	    result.push TkTextMarkInsert.new(self)
+	  when 'current'
+	    result.push TkTextMarkCurrent.new(self)
+	  when 'anchor'
+	    result.push TkTextMarkAnchor.new(self)
+	  else
+	    result.push tk_tcl2rb(val)
+	  end
+	when 'tagon'
+	  if val == 'sel'
+	    if sel
+	      result.push sel
+	    else
+	      result.push TkTextTagSel.new(self)
+	    end
+	  else
+	    result.push tk_tcl2rb val
+	  end
+	when 'tagoff'
+	    result.push tk_tcl2rb sel
+	when 'window'
+	  result.push tk_tcl2rb val
+	end
+	i = idx + 1
+      end
+
+      # retrieve index
+      idx = str.index(/ /, i)
+      if idx
+	result.push str[i..(idx-1)]
+	i = idx + 1
+      else
+	result.push str[i..-1]
+	break
+      end
+    end
+    
+    kvis = []
+    until result.empty?
+      kvis.push [result.shift, result.shift, result.shift]
+    end
+    kvis  # result is [[key1, value1, index1], [key2, value2, index2], ...]
+  end
+
+  def _retrieve_braced_text(str, i)
+    cnt = 0
+    idx = i
+    while idx < str.size
+      case str[idx]
+      when ?{
+	cnt += 1
+      when ?}
+	cnt -= 1
+	if cnt == 0
+	  break
+	end
+      end
+      idx += 1
+    end
+    return str[i+1..idx-1], idx + 2
+  end
+  private :_retrieve_braced_text
+
+  def _retrieve_backslashed_text(str, i)
+    j = i
+    idx = nil
+    loop {
+      idx = str.index(/ /, j)
+      if str[idx-1] == ?\\
+	j += 1
+      else
+	break
+      end
+    }
+    val = str[i..(idx-1)]
+    val.gsub!(/\\( |\{|\})/, '\1')
+    return val, idx + 1
+  end
+  private :_retrieve_backslashed_text
+
+  def dump_all(*index)
+    dump(['all'], *index)
+  end
+  def dump_command(cmd, *index)
+    dump([['command', cmd]], *index)
+  end
+  def dump_mark(*index)
+    dump(['mark'], *index)
+  end
+  def dump_tag(*index)
+    dump(['tag'], *index)
+  end
+  def dump_text(*index)
+    dump(['text'], *index)
+  end
+  def dump_window(*index)
+    dump(['window'], *index)
+  end
+  def dump_image(*index)
+    dump(['image'], *index)
+  end
 end
 
 class TkTextTag<TkObject
   include TkTreatTagFont
 
-  $tk_text_tag = 'tag0000'
+  Tk_TextTag_ID = ['tag0000']
+
   def initialize(parent, *args)
     if not parent.kind_of?(TkText)
       fail format("%s need to be TkText", parent.inspect)
     end
     @parent = @t = parent
-    @path = @id = $tk_text_tag
-    $tk_text_tag = $tk_text_tag.succ
+    @path = @id = Tk_TextTag_ID[0]
+    Tk_TextTag_ID[0] = Tk_TextTag_ID[0].succ
     #tk_call @t.path, "tag", "configure", @id, *hash_kv(keys)
     if args != [] then
       keys = args.pop
@@ -541,14 +706,14 @@ class TkTextTagSel<TkTextTag
 end
 
 class TkTextMark<TkObject
-  $tk_text_mark = 'mark0000'
+  Tk_TextMark_ID = ['mark0000']
   def initialize(parent, index)
     if not parent.kind_of?(TkText)
       fail format("%s need to be TkText", parent.inspect)
     end
     @t = parent
-    @path = @id = $tk_text_mark
-    $tk_text_mark = $tk_text_mark.succ
+    @path = @id = Tk_TextMark_ID[0]
+    Tk_TextMark_ID[0] = Tk_TextMark_ID[0].succ
     tk_call @t.path, 'mark', 'set', @id, index
     @t._addtag id, self
   end
@@ -572,6 +737,14 @@ class TkTextMark<TkObject
   def gravity=(direction)
     tk_call @t.path, 'mark', 'gravity', @id, direction
   end
+
+  def next(index)
+    @t.tagid2obj(tk_call(@t.path, 'mark', 'next', index))
+  end
+
+  def previous(index)
+    @t.tagid2obj(tk_call(@t.path, 'mark', 'previous', index))
+  end
 end
 
 class TkTextMarkInsert<TkTextMark
@@ -706,132 +879,6 @@ class TkTextWindow<TkObject
     end
   end
 
-  def _dump(type, *index)
-    str = tk_send('dump', type, *index)
-    result = []
-    sel = nil
-    i = 0
-    while i < str.size
-      # retrieve key
-      idx = str.index(/ /, i)
-      result.push str[i..(idx-1)]
-      i = idx + 1
-      
-      # retrieve value
-      case result[-1]
-      when 'text'
-	if str[i] == ?{
-	  # text formed as {...}
-	  val, i = _retrieve_braced_text(str, i)
-	  result.push val
-	else
-	  # text which may contain backslahes
-	  val, i = _retrieve_backslashed_text(str, i)
-	  result.push val
-	end
-      else
-	idx = str.index(/ /, i)
-	val = str[i..(idx-1)]
-	case result[-1]
-	when 'mark'
-	  case val
-	  when 'insert'
-	    result.push TkTextMarkInsert.new(self)
-	  when 'current'
-	    result.push TkTextMarkCurrent.new(self)
-	  when 'anchor'
-	    result.push TkTextMarkAnchor.new(self)
-	  else
-	    result.push tk_tcl2rb(val)
-	  end
-	when 'tagon'
-	  if val == 'sel'
-	    if sel
-	      result.push sel
-	    else
-	      result.push TkTextTagSel.new(self)
-	    end
-	  else
-	    result.push tk_tcl2rb val
-	  end
-	when 'tagoff'
-	    result.push tk_tcl2rb sel
-	when 'window'
-	  result.push tk_tcl2rb val
-	end
-	i = idx + 1
-      end
-
-      # retrieve index
-      idx = str.index(/ /, i)
-      if idx
-	result.push str[i..(idx-1)]
-	i = idx + 1
-      else
-	result.push str[i..-1]
-	break
-      end
-    end
-    
-    kvis = []
-    until result.empty?
-      kvis.push [result.shift, result.shift, result.shift]
-    end
-    kvis  # result is [[key1, value1, index1], [key2, value2, index2], ...]
-  end
-  private :_dump
-
-  def _retrieve_braced_text(str, i)
-    cnt = 0
-    idx = i
-    while idx < str.size
-      case str[idx]
-      when ?{
-	cnt += 1
-      when ?}
-	cnt -= 1
-	if cnt == 0
-	  break
-	end
-      end
-      idx += 1
-    end
-    return str[i+1..idx-1], idx + 2
-  end
-  private :_retrieve_braced_text
-
-  def _retrieve_backslashed_text(str, i)
-    j = i
-    idx = nil
-    loop {
-      idx = str.index(/ /, j)
-      if str[idx-1] == ?\\
-	j += 1
-      else
-	break
-      end
-    }
-    val = str[i..(idx-1)]
-    val.gsub!(/\\( |\{|\})/, '\1')
-    return val, idx + 1
-  end
-  private :_retrieve_backslashed_text
-
-  def dump_all(*index)
-    _dump('-all', *index)
-  end
-  def dump_mark(*index)
-    _dump('-mark', *index)
-  end
-  def dump_tag(*index)
-    _dump('-tag', *index)
-  end
-  def dump_text(*index)
-    _dump('-text', *index)
-  end
-  def dump_window(*index)
-    _dump('-window', *index)
-  end
 end
 
 class TkTextImage<TkObject
