@@ -3,10 +3,10 @@
   file.c -
 
   $Author: matz $
-  $Date: 1999/12/08 00:37:33 $
+  $Date: 2000/03/24 09:37:25 $
   created at: Mon Nov 15 12:24:34 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -423,7 +423,7 @@ test_c(obj, fname)
     struct stat st;
 
     if (rb_stat(fname, &st) < 0) return Qfalse;
-    if (S_ISBLK(st.st_mode)) return Qtrue;
+    if (S_ISCHR(st.st_mode)) return Qtrue;
 
     return Qfalse;
 }
@@ -1126,7 +1126,7 @@ rb_file_s_expand_path(argc, argv)
 	p = &buf[strlen(buf)];
 	while (p > buf && *(p - 1) == '/') p--;
     }
-    else if (isdirsep(*s)) {
+    else {
 	while (*s && isdirsep(*s)) {
 	    *p++ = '/';
 	    s++;
@@ -1148,7 +1148,8 @@ rb_file_s_expand_path(argc, argv)
 		    }
 		    else {
 			*++p = '.';
-			*++p = '.';
+			do *++p = '.'; while (*++s == '.');
+			--s;
 		    }
 		    break;
 		  case '/':
@@ -1404,7 +1405,7 @@ test_check(n, argc, argv)
     int i;
 
     n+=1;
-    if (n < argc) rb_raise(rb_eArgError, "wrong # of arguments(%d for %d)", argc, n);
+    if (n != argc) rb_raise(rb_eArgError, "wrong # of arguments(%d for %d)", argc, n);
     for (i=1; i<n; i++) {
 	switch (TYPE(argv[i])) {
 	  case T_STRING:
