@@ -3,10 +3,10 @@
   gc.c -
 
   $Author: matz $
-  $Date: 1999/12/08 05:35:57 $
+  $Date: 2000/04/10 05:48:32 $
   created at: Tue Oct  5 09:44:46 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -50,7 +50,6 @@ static unsigned long malloc_memories = 0;
 static unsigned long alloc_objects = 0;
 
 static int malloc_called = 0;
-static int free_called = 0;
 
 #ifndef xmalloc
 void *
@@ -115,12 +114,11 @@ xrealloc(ptr, size)
     return mem;
 }
 
-static void
+void
 xfree(x)
     void *x;
 {
-    free_called++;
-    free(x);
+    if (x) free(x);
 }
 #endif
 
@@ -370,7 +368,7 @@ rb_mark_tbl(tbl)
 
 static int
 mark_hashentry(key, value)
-    ID key;
+    VALUE key;
     VALUE value;
 {
     rb_gc_mark(key);
@@ -860,6 +858,9 @@ _rb_setjmp:
 	movem.l	d3-d7/a3-a5,(a0)
 	moveq.l	#0,d0
 	rts");
+#ifdef setjmp
+#undef setjmp
+#endif
 #else
 #if defined(DJGPP)
 typedef unsigned long rb_jmp_buf[6];
@@ -1133,8 +1134,10 @@ rb_gc_call_finalizer_at_exit()
     for (i = 0; i < heaps_used; i++) {
 	p = heaps[i]; pend = p + HEAP_SLOTS;
 	while (p < pend) {
-	    if (FL_TEST(p, FL_FINALIZE))
+	    if (FL_TEST(p, FL_FINALIZE)) {
+		p->as.free.flag = 0;
 		run_final((VALUE)p);
+	    }
 	    p++;
 	}
     }
