@@ -3,10 +3,10 @@
   io.c -
 
   $Author: matz $
-  $Date: 1999/11/24 07:13:51 $
+  $Date: 2000/04/10 05:48:36 $
   created at: Fri Oct 15 18:08:59 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -455,7 +455,9 @@ io_fread(ptr, len, f)
 	if (!READ_DATA_PENDING(f)) {
 	    rb_thread_wait_fd(fileno(f));
 	}
+	TRAP_BEG;
 	c = getc(f);
+	TRAP_END;
 	if (c == EOF) {
 	    *ptr = '\0';
 	    break;
@@ -955,7 +957,7 @@ rb_io_close(io)
 {
     OpenFile *fptr;
 
-    GetOpenFile(io, fptr);
+    fptr = RFILE(io)->fptr;
     rb_io_fptr_close(fptr);
     if (fptr->pid) {
 	rb_syswait(fptr->pid);
@@ -1254,6 +1256,9 @@ rb_fopen(fname, mode)
 	    rb_sys_fail(fname);
 	}
     }
+#ifdef __human68k__
+    fmode(file, _IOTEXT);
+#endif
     return file;
 }
 
@@ -1448,7 +1453,8 @@ pipe_open(pname, mode)
 	pipe_add_fptr(fptr);
 	if (modef & FMODE_READABLE) fptr->f  = f;
 	if (modef & FMODE_WRITABLE) {
-	    fptr->f2 = f;
+	    if (fptr->f) fptr->f2 = f;
+	    else fptr->f = f;
 	    rb_io_synchronized(fptr);
 	}
 	return (VALUE)port;
@@ -1582,8 +1588,8 @@ rb_file_s_open(argc, argv, klass)
     path = RSTRING(fname)->ptr;
 
     if (FIXNUM_P(vmode)) {
-	int flags = FIX2INT(vmode);
-	int fmode = NIL_P(perm) ? 0666 : FIX2INT(perm);
+	int flags = NUM2INT(vmode);
+	int fmode = NIL_P(perm) ? 0666 : NUM2INT(perm);
 
 	file = rb_file_sysopen_internal(klass, path, flags, fmode);
     }
@@ -1623,7 +1629,7 @@ rb_f_open(argc, argv)
 	mode = "r";
     }
     else if (FIXNUM_P(pmode)) {
-	mode = rb_io_flags_mode(FIX2INT(pmode));
+	mode = rb_io_flags_mode(NUM2INT(pmode));
     }
     else {
 	int len;
@@ -1851,7 +1857,7 @@ rb_io_printf(argc, argv, out)
     VALUE argv[];
     VALUE out;
 {
-    io_write(out, rb_f_sprintf(argc, argv));
+    rb_io_write(out, rb_f_sprintf(argc, argv));
     return Qnil;
 }
 
@@ -1871,7 +1877,7 @@ rb_f_printf(argc, argv)
 	argv++;
 	argc--;
     }
-    io_write(out, rb_f_sprintf(argc, argv));
+    rb_io_write(out, rb_f_sprintf(argc, argv));
 
     return Qnil;
 }
@@ -2363,7 +2369,7 @@ rb_f_gets_internal(argc, argv)
     if (TYPE(current_file) != T_FILE) {
 	line = rb_funcall3(current_file, rb_intern("gets"), argc, argv);
     }
-    if (argc == 0 && rb_rs == rb_default_rs) {
+    else if (argc == 0 && rb_rs == rb_default_rs) {
 	line = rb_io_gets(current_file);
     }
     else {
@@ -3070,6 +3076,8 @@ argf_eof()
 {
     if (init_p == 0 && !next_argv())
 	return Qtrue;
+    if (next_p == -1)
+	return Qtrue;
     if (TYPE(current_file) != T_FILE) {
 	return argf_forward();
     }
