@@ -1,11 +1,11 @@
 =begin
-$Date: 1999/09/13 23:00:58 $
+$Date: 2000/04/03 18:31:42 $
 
 == CGI SUPPORT LIBRARY
 
-CGI.rb
+cgi.rb
 
-Version 1.00
+Version 1.20
 
 Copyright (C) 1999  Network Applied Communication Laboratory, Inc.
 
@@ -24,7 +24,7 @@ Wakou Aoyama <wakou@fsinet.or.jp>
 
 	# returns true if form has 'field_name'
 	cgi.has_key?('field_name')
-	cgi.key?('field_name')
+	cgi.has_key?('field_name')
 	cgi.include?('field_name')
 
 
@@ -182,11 +182,11 @@ class CGI
   EOL = CR + LF
 v = $-v
 $-v = false
-  VERSION = "1.00"
-  RELEASE_DATE = "$Date: 1999/09/13 23:00:58 $"
+  VERSION = "1.20"
+  RELEASE_DATE = "$Date: 2000/04/03 18:31:42 $"
 $-v = v
 
-  NEEDS_BINMODE = true if /WIN/ni === PLATFORM
+  NEEDS_BINMODE = true if /WIN/ni === RUBY_PLATFORM
   PATH_SEPARATOR = {'UNIX'=>'/', 'WINDOWS'=>'\\', 'MACINTOSH'=>':'}
 
   HTTP_STATUS = {
@@ -237,7 +237,8 @@ $-v = v
 =end
   def CGI::escape(string)
     str = string.dup
-    str.gsub!(/[^a-zA-Z0-9_.-]/n){ sprintf("%%%02X", $&.unpack("C")[0]) }
+    str.gsub!(/ /n, '+')
+    str.gsub!(/([^a-zA-Z0-9_.-])/n){ sprintf("%%%02X", $1.unpack("C")[0]) }
     str
   end
 
@@ -281,7 +282,16 @@ $-v = v
       when /\Aquot\z/ni          then '"'
       when /\Agt\z/ni            then '>'
       when /\Alt\z/ni            then '<'
-      when /\A#(\d+)\z/n         then Integer($1).chr
+      when /\A#(\d+)\z/n         then
+        if Integer($1) < 256
+          Integer($1).chr
+        else
+          if $KCODE[0] == ?u or $KCODE[0] == ?U
+            [Integer($1)].pack("U")
+          else
+            "#" + $1
+          end
+        end
       when /\A#x([0-9a-f]+)\z/ni then $1.hex.chr
       end
     }
@@ -391,11 +401,11 @@ status:
       options = { "type" => options }
     end
 
-    unless options.key?("type")
+    unless options.has_key?("type")
       options["type"] = "text/html"
     end
 
-    if options.key?("charset")
+    if options.has_key?("charset")
       options["type"].concat( "; charset=" )
       options["type"].concat( options.delete("charset") )
     end
@@ -411,40 +421,40 @@ status:
         "Date: " + CGI::rfc1123_date(Time.now) + EOL
       )
 
-      unless options.key?("server")
+      unless options.has_key?("server")
         options["server"] = (env_table['SERVER_SOFTWARE'] or "")
       end
 
-      unless options.key?("connection")
+      unless options.has_key?("connection")
         options["connection"] = "close"
       end
 
     end
     options.delete("status")
 
-    if options.key?("server")
+    if options.has_key?("server")
       buf.concat("Server: " + options.delete("server") + EOL)
     end
 
-    if options.key?("connection")
+    if options.has_key?("connection")
       buf.concat("Connection: " + options.delete("connection") + EOL)
     end
 
     buf.concat("Content-Type: " + options.delete("type") + EOL)
 
-    if options.key?("length")
+    if options.has_key?("length")
       buf.concat("Content-Length: " + options.delete("length").to_s + EOL)
     end
 
-    if options.key?("language")
+    if options.has_key?("language")
       buf.concat("Content-Language: " + options.delete("language") + EOL)
     end
 
-    if options.key?("expires")
+    if options.has_key?("expires")
       buf.concat("Expires: " + CGI::rfc1123_date( options.delete("expires") ) + EOL)
     end
 
-    if options.key?("cookie")
+    if options.has_key?("cookie")
       if options["cookie"].kind_of?(String) or
            options["cookie"].kind_of?(Cookie)
         buf.concat("Set-Cookie: " + options.delete("cookie").to_s + EOL)
@@ -468,7 +478,7 @@ status:
       buf.concat(key + ": " + value + EOL)
     }
 
-    if env_table['MOD_RUBY']
+    if defined?(MOD_RUBY)
       buf.scan(/([^:]+): (.+)#{EOL}/n){
         Apache::request[$1] = $2
       }
@@ -518,18 +528,18 @@ convert string charset, and set language to "ja".
     options = { "type" => options } if options.kind_of?(String)
     content = yield
 
-    if options.key?("charset")
+    if options.has_key?("charset")
       require "nkf"
       case options["charset"]
       when /iso-2022-jp/ni
         content = NKF::nkf('-j', content)
-        options["language"] = "ja" unless options.key?("language")
+        options["language"] = "ja" unless options.has_key?("language")
       when /euc-jp/ni
         content = NKF::nkf('-e', content)
-        options["language"] = "ja" unless options.key?("language")
+        options["language"] = "ja" unless options.has_key?("language")
       when /shift_jis/ni
         content = NKF::nkf('-s', content)
-        options["language"] = "ja" unless options.key?("language")
+        options["language"] = "ja" unless options.has_key?("language")
       end
     end
 
@@ -588,13 +598,23 @@ convert string charset, and set language to "ja".
                 else
                   name
                 end
-      unless options.key?("name")
+      unless options.has_key?("name")
         raise ArgumentError, "`name' required"
       end
 
       @name = options["name"]
       @value = Array(options["value"])
-      @path = options["path"]
+      # simple support for IE
+      if options["path"]
+        @path = options["path"]
+      elsif ENV["REQUEST_URI"]
+        @path = ENV["REQUEST_URI"].sub(/\?.*/n,'')
+        if ENV["PATH_INFO"]
+          @path = @path[0...@path.rindex(ENV["PATH_INFO"])]
+        end
+      else
+        @path = ENV["SCRIPT_NAME"] or ""
+      end
       @domain = options["domain"]
       @expires = options["expires"]
       @secure = options["secure"] == true ? true : false
@@ -648,12 +668,14 @@ convert string charset, and set language to "ja".
 =end
   def Cookie::parse(raw_cookie)
     cookies = Hash.new([])
+    return cookies unless raw_cookie
 
     raw_cookie.split('; ').each do |pairs|
       name, values = pairs.split('=',2)
       name = CGI::unescape(name)
+      values ||= ""
       values = values.split('&').filter{|v| CGI::unescape(v) }
-      if cookies.key?(name)
+      if cookies.has_key?(name)
         cookies[name].value.push(*values)
       else
         cookies[name] = Cookie::new({ "name" => name, "value" => values })
@@ -675,7 +697,7 @@ convert string charset, and set language to "ja".
 
     query.split(/[&;]/n).each do |pairs|
       key, value = pairs.split('=',2).filter{|v| CGI::unescape(v) }
-      if params.key?(key)
+      if params.has_key?(key)
         params[key].push(value)
       else
         params[key] = [value]
@@ -812,7 +834,7 @@ convert string charset, and set language to "ja".
         /Content-Disposition:.* name="?([^\";]*)"?/ni === head
         name = $1.dup
 
-        if params.key?(name)
+        if params.has_key?(name)
           params[name].push(body)
         else
           params[name] = [body]
@@ -863,7 +885,7 @@ convert string charset, and set language to "ja".
         @params = CGI::parse(
                     case env_table['REQUEST_METHOD']
                     when "GET", "HEAD"
-                      if env_table['MOD_RUBY']
+                      if defined?(MOD_RUBY)
                         Apache::request.args or ""
                       else
                         env_table['QUERY_STRING'] or ""
@@ -877,8 +899,7 @@ convert string charset, and set language to "ja".
                   )
       end
 
-      @cookies = CGI::Cookie::parse((env_table['HTTP_COOKIE'] or
-        env_table['COOKIE'] or ""))
+      @cookies = CGI::Cookie::parse((env_table['HTTP_COOKIE'] or env_table['COOKIE']))
 
     end
     private :initialize_query
@@ -959,6 +980,7 @@ convert string charset, and set language to "ja".
     def nn_element_def(element)
       <<-END.gsub(/element\.downcase/n, element.downcase).gsub(/element\.upcase/n, element.upcase)
           "<element.upcase" + attributes.collect{|name, value|
+            next unless value
             " " + CGI::escapeHTML(name) +
             if true == value
               ""
@@ -979,6 +1001,7 @@ convert string charset, and set language to "ja".
     def nOE_element_def(element)
       <<-END.gsub(/element\.downcase/n, element.downcase).gsub(/element\.upcase/n, element.upcase)
           "<element.upcase" + attributes.collect{|name, value|
+            next unless value
             " " + CGI::escapeHTML(name) +
             if true == value
               ""
@@ -992,7 +1015,8 @@ convert string charset, and set language to "ja".
     # O O or - O
     def nO_element_def(element)
       <<-END.gsub(/element\.downcase/n, element.downcase).gsub(/element\.upcase/n, element.upcase)
-         "<element.upcase" + attributes.collect{|name, value|
+          "<element.upcase" + attributes.collect{|name, value|
+            next unless value
             " " + CGI::escapeHTML(name) +
             if true == value
               ""
@@ -1207,10 +1231,10 @@ convert string charset, and set language to "ja".
                      { "METHOD" => method, "ACTION" => action,
                        "ENCTYPE" => enctype } 
                    else
-                     unless method.key?("METHOD")
+                     unless method.has_key?("METHOD")
                        method["METHOD"] = method
                      end
-                     unless method.key?("ENCTYPE")
+                     unless method.has_key?("ENCTYPE")
                        method["ENCTYPE"] = enctype
                      end
                      method
@@ -1295,7 +1319,7 @@ convert string charset, and set language to "ja".
       pretty = attributes.delete("PRETTY")
       buf = ""
 
-      if attributes.key?("DOCTYPE")
+      if attributes.has_key?("DOCTYPE")
         if attributes["DOCTYPE"]
           buf.concat( attributes.delete("DOCTYPE") )
         else
@@ -1336,8 +1360,9 @@ convert string charset, and set language to "ja".
                      { "TYPE" => "image", "SRC" => src, "NAME" => name,
                        "ALT" => alt }
                    else
-                     name["TYPE"] = "image"
-                     name
+                     src["TYPE"] = "image"
+                     src["SRC"] ||= ""
+                     src
                    end
       input(attributes)
     end
@@ -1378,10 +1403,10 @@ convert string charset, and set language to "ja".
                      { "METHOD" => "post", "ACTION" => action,
                        "ENCTYPE" => enctype } 
                    else
-                     unless action.key?("METHOD")
+                     unless action.has_key?("METHOD")
                        action["METHOD"] = "post"
                      end
-                     unless action.key?("ENCTYPE")
+                     unless action.has_key?("ENCTYPE")
                        action["ENCTYPE"] = enctype
                      end
                      action
@@ -1835,8 +1860,16 @@ convert string charset, and set language to "ja".
 
   def initialize(type = "query")
     extend QueryExtension
-    initialize_query()
-    # @params, @cookies initialized in initialize_query
+    if defined?(CGI_PARAMS)
+      @params  = CGI_PARAMS.nil?  ? nil : CGI_PARAMS.dup
+      @cookies = CGI_COOKIES.nil? ? nil : CGI_COOKIES.dup
+    else
+      initialize_query()  # set @params, @cookies
+      eval "CGI_PARAMS  = @params.nil?  ? nil : @params.dup"
+      eval "CGI_COOKIES = @cookies.nil? ? nil : @cookies.dup"
+    end
+    @output_cookies = nil
+    @output_hidden = nil
 
     case type
     when "html3"
@@ -1853,12 +1886,49 @@ convert string charset, and set language to "ja".
       extend HtmlExtension
     end
   end
+
+  if defined?(MOD_RUBY) and (RUBY_VERSION < "1.4.3")
+    raise "Please, use ruby1.4.3 or later."
+  else
+    at_exit() do
+      if defined?(CGI_PARAMS)
+        remove_const(:CGI_PARAMS)
+        remove_const(:CGI_COOKIES)
+      end
+    end
+  end
 end
 
 
 =begin
 
-== HISTRY
+== HISTORY
+
+=== Version 1.20 - wakou
+
+2000/04/03 18:31:42
+
+- bug fix: CGI#image_button() can't get Hash option
+  thanks to Takashi Ikeda <ikeda@auc.co.jp>
+- CGI::unescapeHTML()
+  simple support for "&#12345;"
+- CGI::Cookie::new()
+  simple support for IE
+- CGI::escape()
+  ' ' replaced by '+'
+
+=== Version 1.10 - wakou
+
+1999/12/06 20:16:34
+
+- can make many CGI objects.
+- if use mod_ruby, then require ruby1.4.3 or later.
+
+=== Version 1.01 - wakou
+
+1999/11/29 21:35:58
+
+- support for ruby 1.5.0 (1999-11-20)
 
 === Version 1.00 - wakou
 
