@@ -2,11 +2,11 @@
 ;;;  ruby-mode.el -
 ;;;
 ;;;  $Author: matz $
-;;;  $Date: 1999/11/29 06:31:06 $
+;;;  $Date: 2000/01/24 03:46:14 $
 ;;;  created at: Fri Feb  4 14:49:13 JST 1994
 ;;;
 
-(defconst ruby-mode-revision "$Revision: 1.5.2.2 $")
+(defconst ruby-mode-revision "$Revision: 1.5.2.3 $")
 
 (defconst ruby-mode-version
   (progn
@@ -180,7 +180,8 @@ The variable ruby-indent-level controls the amount of indentation.
 	(back-to-indentation)
 	(setq top (current-column))
 	(skip-chars-backward " \t")
-	(if (>= shift top) (setq shift (- shift top)))
+	(if (>= shift top) (setq shift (- shift top))
+	  (setq shift 0))
 	(if (and (bolp)
 		 (= x top))
 	    (move-to-column (+ x shift))
