@@ -3,10 +3,10 @@
   node.h -
 
   $Author: matz $
-  $Date: 1999/10/25 05:09:12 $
+  $Date: 2000/03/23 04:14:19 $
   created at: Fri May 28 15:14:02 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -299,7 +299,7 @@ typedef struct RNode {
 #define NEW_MODULE(n,b) rb_node_newnode(NODE_MODULE,n,NEW_CBODY(b),0)
 #define NEW_COLON2(c,i) rb_node_newnode(NODE_COLON2,c,i,0)
 #define NEW_COLON3(i) rb_node_newnode(NODE_COLON3,0,i,0)
-#define NEW_CREF0() (cur_cref=rb_node_newnode(NODE_CREF,RNODE(ruby_frame->cbase)->nd_clss,0,0))
+#define NEW_CREF0() (cur_cref=RNODE(ruby_frame->cbase))
 #define NEW_CREF() (cur_cref=rb_node_newnode(NODE_CREF,0,0,cur_cref))
 #define NEW_CBODY(b) (cur_cref->nd_body=NEW_SCOPE(b),cur_cref)
 #define NEW_DOT2(b,e) rb_node_newnode(NODE_DOT2,b,e,0)
@@ -314,9 +314,6 @@ typedef struct RNode {
 #define NEW_PREEXE(b) NEW_SCOPE(b)
 #define NEW_POSTEXE() rb_node_newnode(NODE_POSTEXE,0,0,0)
 
-NODE *rb_node_newnode();
-VALUE rb_method_booundp();
-
 #define NOEX_PUBLIC    0
 #define NOEX_UNDEF     1
 #define NOEX_CFUNC     1
