@@ -3,10 +3,10 @@
   numeric.c -
 
   $Author: matz $
-  $Date: 1999/12/06 09:03:32 $
+  $Date: 2000/02/17 08:28:58 $
   created at: Fri Aug 13 18:33:09 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -43,8 +43,8 @@ num_coerce(x, y)
     VALUE x, y;
 {
     if (CLASS_OF(x) == CLASS_OF(y))
-	return rb_assoc_new(x, y);
-    return rb_assoc_new(rb_Float(x), rb_Float(y));
+	return rb_assoc_new(y, x);
+    return rb_assoc_new(rb_Float(y), rb_Float(x));
 }
 
 static VALUE
@@ -330,8 +330,7 @@ flo_modulo(x, y, modulo)
 	result = value1 - value2 * value;
     }
 #endif
-    if (modulo &&
-	(RFLOAT(x)->value < 0.0) != (result < 0.0) && result != 0.0) {
+    if (modulo && value*result<0.0) {
 	result += value;
     }
     return rb_float_new(result);
