@@ -3,10 +3,10 @@
   random.c -
 
   $Author: matz $
-  $Date: 1999/12/06 09:03:33 $
+  $Date: 2000/02/08 08:53:55 $
   created at: Fri Dec 24 16:39:21 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -76,11 +76,35 @@ void srand48 _((long));
 
 #endif /* not HAVE_DRAND48 */
 
-#ifdef HAVE_RANDOM
 static int first = 1;
+#ifdef HAVE_RANDOM
 static char state[256];
 #endif
 
+static int
+rand_init(seed)
+    long seed;
+{
+    int old;
+    static unsigned int saved_seed;
+
+#ifdef HAVE_RANDOM
+    if (first == 1) {
+	initstate(1, state, sizeof state);
+    }
+    else {
+	setstate(state);
+    }
+#endif
+    first = 0;
+
+    SRANDOM(seed);
+    old = saved_seed;
+    saved_seed = seed;
+
+    return old;
+}
+
 static VALUE
 rb_f_srand(argc, argv, obj)
     int argc;
@@ -89,7 +113,6 @@ rb_f_srand(argc, argv, obj)
 {
     VALUE a;
     unsigned int seed, old;
-    static unsigned int saved_seed;
 
     if (rb_scan_args(argc, argv, "01", &a) == 0) {
 	static int n = 0;
@@ -101,20 +124,7 @@ rb_f_srand(argc, argv, obj)
     else {
 	seed = NUM2UINT(a);
     }
-
-#ifdef HAVE_RANDOM
-    if (first == 1) {
-	initstate(1, state, sizeof state);
-	first = 0;
-    }
-    else {
-	setstate(state);
-    }
-#endif
-
-    SRANDOM(seed);
-    old = saved_seed;
-    saved_seed = seed;
+    old = rand_init(seed);
 
     return rb_uint2inum(old);
 }
@@ -125,6 +135,12 @@ rb_f_rand(obj, vmax)
 {
     long val, max;
 
+    if (first) {
+	struct timeval tv;
+
+	gettimeofday(&tv, 0);
+	rand_init(tv.tv_sec ^ tv.tv_usec ^ getpid());
+    }
     switch (TYPE(vmax)) {
       case T_FLOAT:
 	if (RFLOAT(vmax)->value <= LONG_MAX && RFLOAT(vmax)->value >= LONG_MIN)
