@@ -3,10 +3,10 @@
   ruby.c -
 
   $Author: matz $
-  $Date: 1999/10/15 08:44:37 $
+  $Date: 2000/03/23 04:14:20 $
   created at: Tue Aug 10 12:47:31 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -76,7 +76,7 @@ usage(name)
     const char *name;
 {
     /* This message really ought to be max 23 lines.
-     * Removed -h because the user already knows that opton. Others? */
+     * Removed -h because the user already knows that option. Others? */
 
     static char *usage_msg[] = {
 "-0[octal]       specify record separator (\\0, if no argument)",
@@ -111,13 +111,6 @@ NULL
 	printf("\n  %s", *p++);
 }
 
-#ifndef RUBY_LIB
-#define RUBY_LIB "/usr/local/lib/ruby"
-#endif
-#ifndef RUBY_SITE_LIB
-#define RUBY_SITE_LIB "/usr/local/lib/site_ruby"
-#endif
-
 extern VALUE rb_load_path;
 
 static FILE *e_fp;
@@ -392,7 +385,10 @@ proc_options(argcp, argvp)
 		s = argv[1];
 		argc--,argv++;
 	    }
-	    if (*s && chdir(s) < 0) {
+	    if (!s || !*s) {
+		rb_fatal("Can't chdir");
+	    }
+	    if (chdir(s) < 0) {
 		rb_fatal("Can't chdir to %s", s);
 	    }
 	    break;
@@ -560,10 +556,10 @@ proc_options(argcp, argvp)
 	    argv[0][0] = '$';
 	    if (s = strchr(argv[0], '=')) {
 		*s++ = '\0';
-		rb_gvar_set2(argv[0], rb_str_new2(s));
+		rb_gv_set(argv[0], rb_str_new2(s));
 	    }
 	    else {
-		rb_gvar_set2(argv[0], Qtrue);
+		rb_gv_set(argv[0], Qtrue);
 	    }
 	    argv[0][0] = '-';
 	}
@@ -680,7 +676,11 @@ load_file(fname, script)
 		}
 	    }
 	}
-	else if (!NIL_P(c)) {
+	else if (NIL_P(c)) {
+	    rb_io_close(f);
+	    return;
+	}
+	else {
 	    rb_io_ungetc(f, c);
 	}
     }
@@ -865,17 +865,14 @@ ruby_prog_init()
     addpath(ruby_libpath());
 #endif
 
-#ifdef RUBY_ARCHLIB
     addpath(RUBY_ARCHLIB);
-#endif
 #ifdef RUBY_THIN_ARCHLIB
     addpath(RUBY_THIN_ARCHLIB);
 #endif
 
     addpath(RUBY_SITE_LIB);
-#ifdef RUBY_SITE_ARCHLIB
+    addpath(RUBY_SITE_LIB2);
     addpath(RUBY_SITE_ARCHLIB);
-#endif
 #ifdef RUBY_SITE_THIN_ARCHLIB
     addpath(RUBY_SITE_THIN_ARCHLIB);
 #endif
