@@ -5,7 +5,7 @@
   $Author: matz $
   created at: Thu Jun 10 14:26:32 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 *************************************************/
 
@@ -89,7 +89,7 @@ extern "C" {
 ---->> ruby requires sizeof(void*) == sizeof(long) to be compiled. <<----
 #endif
 typedef unsigned long VALUE;
-typedef unsigned int ID;
+typedef unsigned long ID;
 
 #ifdef __STDC__
 # include <limits.h>
@@ -117,6 +117,8 @@ typedef unsigned int ID;
 #define INT2FIX(i) (VALUE)(((long)(i))<<1 | FIXNUM_FLAG)
 VALUE rb_int2inum _((long));
 #define INT2NUM(v) rb_int2inum(v)
+VALUE rb_uint2inum _((unsigned long));
+#define UINT2NUM(v) rb_uint2inum(v)
 
 #define FIX2LONG(x) RSHIFT((long)x,1)
 #define FIX2ULONG(x) (((unsigned long)(x))>>1)
@@ -190,6 +192,8 @@ int rb_fix2int _((VALUE));
 #define FIX2INT(x) FIX2LONG(x)
 #define FIX2UINT(x) FIX2ULONG(x)
 #endif
+#define ID2SYM(x) INT2FIX(x)
+#define SYM2ID(x) FIX2INT(x)
 
 double rb_num2dbl _((VALUE));
 #define NUM2DBL(x) rb_num2dbl((VALUE)(x))
@@ -209,10 +213,11 @@ VALUE rb_newobj _((void));
     RBASIC(obj)->flags = (t);\
     if (rb_safe_level() >= 3) FL_SET(obj, FL_TAINT);\
 }
-#define CLONESETUP(clone,obj) {\
+#define CLONESETUP(clone,obj) do {\
     OBJSETUP(clone,rb_singleton_class_clone(RBASIC(obj)->klass),RBASIC(obj)->flags);\
     rb_singleton_class_attached(RBASIC(clone)->klass, (VALUE)clone);\
-}
+    if (FL_TEST(obj, FL_EXIVAR)) rb_clone_generic_ivar((VALUE)clone,(VALUE)obj);\
+} while (0)
 
 struct RBasic {
     unsigned long flags;
@@ -397,8 +402,12 @@ VALUE rb_eval_string _((const char*));
 VALUE rb_eval_string_protect _((const char*, int*));
 VALUE rb_eval_string_wrap _((const char*, int*));
 VALUE rb_funcall __((VALUE, ID, int, ...));
+VALUE rb_funcall2 _((VALUE, ID, int, VALUE*));
+VALUE rb_funcall3 _((VALUE, ID, int, VALUE*));
 int rb_scan_args __((int, VALUE*, const char*, ...));
 
+VALUE rb_gv_set _((const char*, VALUE));
+VALUE rb_gv_get _((const char*));
 VALUE rb_iv_get _((VALUE, const char*));
 VALUE rb_iv_set _((VALUE, const char*, VALUE));
 VALUE rb_const_get _((VALUE, ID));
