@@ -1,9 +1,10 @@
 line = ''
 indent=0
+$stdout.sync = TRUE
 print "ruby> "
 while TRUE
   l = gets
-  if not l
+  unless l
     break if line == ''
   else
     line = line + l 
@@ -11,13 +12,13 @@ while TRUE
       print "ruby| "
       next
     end
-    if l =~ /^\s*(class|module|def|if|case|while|for|begin)\b[^_]/
+    if l =~ /^\s*(class|module|def|if|unless|case|while|until|for|begin)\b[^_]/
       indent += 1
     end
     if l =~ /^\s*end\b[^_]/
       indent -= 1
     end
-    if l =~ /{\s*(\|.*\|)?\s*$/
+    if l =~ /\{\s*(\|.*\|)?\s*$/
       indent += 1
     end
     if l =~ /^\s*\}/
@@ -31,7 +32,7 @@ while TRUE
   begin
     print eval(line).inspect, "\n"
   rescue
-    $! = 'exception raised' if not $!
+    $! = 'exception raised' unless $!
     print "ERR: ", $!, "\n"
   end
   break if not l
