@@ -3,10 +3,10 @@
   string.c -
 
   $Author: matz $
-  $Date: 1999/12/01 09:24:46 $
+  $Date: 2000/03/23 04:14:21 $
   created at: Mon Aug  9 17:12:58 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -262,9 +262,13 @@ rb_str_times(str, times)
     long i, len;
 
     len = NUM2LONG(times);
+    if (len == 0) return rb_str_new(0,0);
     if (len < 0) {
 	rb_raise(rb_eArgError, "negative argument");
     }
+    if (LONG_MAX / len <  RSTRING(str)->len) {
+	rb_raise(rb_eArgError, "argument too big");
+    }
 
     str2 = rb_str_new(0, RSTRING(str)->len*len);
     for (i=0; i<len; i++) {
@@ -580,6 +584,10 @@ rb_str_index_method(argc, argv, str)
     else {
 	pos = 0;
     }
+    if (pos < 0) {
+	pos += RSTRING(str)->len;
+	if (pos < 0) return Qnil;
+    }
 
     switch (TYPE(sub)) {
       case T_REGEXP:
@@ -860,7 +868,7 @@ rb_str_replace(str, beg, len, val)
     if (RSTRING(str)->len < beg && len < 0) {
 	MEMZERO(RSTRING(str)->ptr + RSTRING(str)->len, char, -len);
     }
-    memcpy(RSTRING(str)->ptr+beg, RSTRING(val)->ptr, RSTRING(val)->len);
+    memmove(RSTRING(str)->ptr+beg, RSTRING(val)->ptr, RSTRING(val)->len);
     RSTRING(str)->len += RSTRING(val)->len - len;
     RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
 }
@@ -1043,10 +1051,9 @@ rb_str_sub(argc, argv, str)
     VALUE *argv;
     VALUE str;
 {
-    VALUE val = rb_str_sub_bang(argc, argv, str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_sub_bang(argc, argv, str);
+    return str;
 }
 
 static VALUE
@@ -1150,10 +1157,10 @@ rb_str_gsub(argc, argv, str)
     VALUE *argv;
     VALUE str;
 {
-    VALUE val = rb_str_gsub_bang(argc, argv, str = rb_str_dup(str));
+    str = rb_str_dup(str);
 
-    if (NIL_P(val)) return str;
-    return val;
+    rb_str_gsub_bang(argc, argv, str);
+    return str;
 }
 
 static VALUE
@@ -1195,13 +1202,11 @@ rb_f_sub(argc, argv)
     int argc;
     VALUE *argv;
 {
-    VALUE line, v;
+    VALUE str = rb_str_dup(uscore_get());
 
-    line = uscore_get();
-    v = rb_str_sub_bang(argc, argv, line = rb_str_dup(line));
-    if (NIL_P(v)) return line;
-    rb_lastline_set(v);
-    return v;
+    rb_str_sub_bang(argc, argv, str);
+    rb_lastline_set(str);
+    return str;
 }
 
 static VALUE
@@ -1217,13 +1222,11 @@ rb_f_gsub(argc, argv)
     int argc;
     VALUE *argv;
 {
-    VALUE line, v;
+    VALUE str = rb_str_dup(uscore_get());
 
-    line = uscore_get();
-    v = rb_str_gsub_bang(argc, argv, line = rb_str_dup(line));
-    if (NIL_P(v)) return line;
-    rb_lastline_set(v);
-    return v;
+    rb_str_gsub_bang(argc, argv, str);
+    rb_lastline_set(str);
+    return str;
 }
 
 static VALUE
@@ -1518,10 +1521,9 @@ static VALUE
 rb_str_upcase(str)
     VALUE str;
 {
-    VALUE val = rb_str_upcase_bang(str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_upcase_bang(str);
+    return str;
 }
 
 static VALUE
@@ -1552,10 +1554,9 @@ static VALUE
 rb_str_downcase(str)
     VALUE str;
 {
-    VALUE val = rb_str_downcase_bang(str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_downcase_bang(str);
+    return str;
 }
 
 static VALUE
@@ -1588,10 +1589,9 @@ static VALUE
 rb_str_capitalize(str)
     VALUE str;
 {
-    VALUE val = rb_str_capitalize_bang(str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_capitalize_bang(str);
+    return str;
 }
 
 static VALUE
@@ -1626,10 +1626,9 @@ static VALUE
 rb_str_swapcase(str)
     VALUE str;
 {
-    VALUE val = rb_str_swapcase_bang(str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_swapcase_bang(str);
+    return str;
 }
 
 typedef unsigned char *USTR;
@@ -1777,10 +1776,9 @@ static VALUE
 rb_str_tr(str, src, repl)
     VALUE str, src, repl;
 {
-    VALUE val = tr_trans(str = rb_str_dup(str), src, repl, 0);
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    tr_trans(str, src, repl, 0);
+    return str;
 }
 
 static void
@@ -1861,10 +1859,9 @@ rb_str_delete(argc, argv, str)
     VALUE *argv;
     VALUE str;
 {
-    VALUE val = rb_str_delete_bang(argc, argv, str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_delete_bang(argc, argv, str);
+    return str;
 }
 
 static VALUE
@@ -1922,10 +1919,9 @@ rb_str_squeeze(argc, argv, str)
     VALUE *argv;
     VALUE str;
 {
-    VALUE val = rb_str_squeeze_bang(argc, argv, str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_squeeze_bang(argc, argv, str);
+    return str;
 }
 
 static VALUE
@@ -1939,10 +1935,9 @@ static VALUE
 rb_str_tr_s(str, src, repl)
     VALUE str, src, repl;
 {
-    VALUE val = tr_trans(str = rb_str_dup(str), src, repl, 1);
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    tr_trans(str, src, repl, 1);
+    return str;
 }
 
 static VALUE
@@ -2231,10 +2226,9 @@ static VALUE
 rb_str_chop(str)
     VALUE str;
 {
-    VALUE val = rb_str_chop_bang(str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_chop_bang(str);
+    return str;
 }
 
 static VALUE
@@ -2247,12 +2241,11 @@ rb_f_chop_bang(str)
 static VALUE
 rb_f_chop()
 {
-    VALUE str = uscore_get();
-    VALUE val = rb_str_chop_bang(str = rb_str_dup(str));
+    VALUE str = rb_str_dup(uscore_get());
 
-    if (NIL_P(val)) return str;
-    rb_lastline_set(val);
-    return val;
+    rb_str_chop_bang(str);
+    rb_lastline_set(str);
+    return str;
 }
 
 static VALUE
@@ -2279,6 +2272,7 @@ rb_str_chomp_bang(argc, argv, str)
 	    len--;
 	}
 	if (len < RSTRING(str)->len) {
+	    rb_str_modify(str);
 	    RSTRING(str)->len = len;
 	    RSTRING(str)->ptr[len] = '\0';
 	    return str;
@@ -2291,6 +2285,7 @@ rb_str_chomp_bang(argc, argv, str)
     if (p[len-1] == newline &&
 	(rslen <= 1 ||
 	 memcmp(RSTRING(rs)->ptr, p+len-rslen, rslen) == 0)) {
+	rb_str_modify(str);
 	RSTRING(str)->len -= rslen;
 	RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
 	return str;
@@ -2304,10 +2299,9 @@ rb_str_chomp(argc, argv, str)
     VALUE *argv;
     VALUE str;
 {
-    VALUE val = rb_str_chomp_bang(argc, argv, str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_chomp_bang(argc, argv, str);
+    return str;
 }
 
 static VALUE
@@ -2323,12 +2317,11 @@ rb_f_chomp(argc, argv)
     int argc;
     VALUE *argv;
 {
-    VALUE str = uscore_get();
-    VALUE val = rb_str_chomp_bang(argc, argv, str = rb_str_dup(str));
+    VALUE str = rb_str_dup(uscore_get());
 
-    if (NIL_P(val)) return str;
-    rb_lastline_set(val);
-    return val;
+    rb_str_chomp_bang(argc, argv, str);
+    rb_lastline_set(str);
+    return str;
 }
 
 static VALUE
@@ -2371,10 +2364,9 @@ static VALUE
 rb_str_strip(str)
     VALUE str;
 {
-    VALUE val = rb_str_strip_bang(str = rb_str_dup(str));
-
-    if (NIL_P(val)) return str;
-    return val;
+    str = rb_str_dup(str);
+    rb_str_strip_bang(str);
+    return str;
 }
 
 static VALUE
