@@ -3,10 +3,10 @@
   time.c -
 
   $Author: matz $
-  $Date: 1999/10/04 04:51:06 $
+  $Date: 2000/02/17 08:29:04 $
   created at: Tue Dec 28 14:31:59 JST 1993
 
-  Copyright (C) 1993-1999 Yukihiro Matsumoto
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -386,7 +386,7 @@ time_cmp(time1, time2)
 	i = FIX2LONG(time2);
 	if (tobj1->tv.tv_sec == i) return INT2FIX(0);
 	if (tobj1->tv.tv_sec > i) return INT2FIX(1);
-	return FIX2INT(-1);
+	return INT2FIX(-1);
 	
       case T_FLOAT:
 	{
@@ -399,7 +399,7 @@ time_cmp(time1, time2)
 		return INT2FIX(0);
 	    if (tobj1->tv.tv_sec > (time_t)RFLOAT(time2)->value)
 		return INT2FIX(1);
-	    return FIX2INT(-1);
+	    return INT2FIX(-1);
 	}
     }
 
@@ -408,15 +408,15 @@ time_cmp(time1, time2)
 	if (tobj1->tv.tv_sec == tobj2->tv.tv_sec) {
 	    if (tobj1->tv.tv_usec == tobj2->tv.tv_usec) return INT2FIX(0);
 	    if (tobj1->tv.tv_usec > tobj2->tv.tv_usec) return INT2FIX(1);
-	    return FIX2INT(-1);
+	    return INT2FIX(-1);
 	}
 	if (tobj1->tv.tv_sec > tobj2->tv.tv_sec) return INT2FIX(1);
-	return FIX2INT(-1);
+	return INT2FIX(-1);
     }
     i = NUM2LONG(time2);
     if (tobj1->tv.tv_sec == i) return INT2FIX(0);
     if (tobj1->tv.tv_sec > i) return INT2FIX(1);
-    return FIX2INT(-1);
+    return INT2FIX(-1);
 }
 
 static VALUE
